#import <Foundation/NSJSONSerialization.h>
#import <unistd.h>

#import "PBXCommon.h"
#import "PBXGroup.h"
#import "PBXResourcesBuildPhase.h"
#import "PBXFileReference.h"
#import "PBXBuildFile.h"
#import "PBXVariantGroup.h"
#import "NSString+PBXAdditions.h"
#import "GSXCBuildContext.h"
#import "XCBuildConfiguration.h"
#import "XCConfigurationList.h"

extern char **environ;

@implementation PBXResourcesBuildPhase
- (instancetype) init
{
  self = [super init];
  if (self != nil)
    {
      NSArray *objs = nil;
      objs = [[[GSXCBuildContext sharedBuildContext]
                objectForKey: @"objects"]
               allValues];
      
      ASSIGNCOPY(files, objs);
    }
  return self;
}

- (NSString *) productName
{
  GSXCBuildContext *context = [GSXCBuildContext sharedBuildContext];
  NSDictionary *ctx = [context currentContext];
  XCConfigurationList *xcl = [ctx objectForKey: @"buildConfig"];
  XCBuildConfiguration *xbc = [xcl defaultConfiguration];
  NSDictionary *bs = [xbc buildSettings];
  NSString *productName = [bs objectForKey: @"PRODUCT_NAME"];

  // NSProcessInfo *info = [NSProcessInfo processInfo];
  // NSDictionary *env = [info environment];
  NSDebugLog(@"bs = %@", bs);
  
  // This is kind of a kludge, but better than what was here before.
  // I believe that when the context has the variable name it means to use
  // the product name from the target.
  if ([productName isEqualToString: @"$(TARGET_NAME)"])
    {
      productName = [target productName];
      NSDebugLog(@"* 2nd try %@", productName);
      if ([productName isEqualToString: @"$(TARGET_NAME)"])
	{
	  productName = [target name];
	  NSDebugLog(@"* 3rd try %@", productName);
	}
    }
  
  return productName;
}

- (NSString *) processAssets
{
  NSFileManager *mgr = [NSFileManager defaultManager];
  char cwd[PATH_MAX];
  if (getcwd(cwd, sizeof(cwd)) != NULL)
    {
      NSDebugLog(@"Current working dir is: %s", cwd);
    }
  
  NSString *filename = nil;
  NSString *productName = [self productName];
  NSString *assetsDir = [productName stringByAppendingPathComponent: @"Assets.xcassets"]; 
  NSString *appIconDir = [assetsDir stringByAppendingPathComponent: @"AppIcon.appiconset"];
  NSString *contentsJson = [appIconDir stringByAppendingPathComponent: @"Contents.json"];
  NSData *data = [NSData dataWithContentsOfFile: contentsJson];
  NSDictionary *dict = [NSJSONSerialization JSONObjectWithData: data
                                                       options: 0L
                                                         error: NULL];
  NSArray *imagesArray = [dict objectForKey: @"images"];
  NSDictionary *imageDict = nil;
  NSEnumerator *en = [imagesArray objectEnumerator];
  
  while ((imageDict = [en nextObject]) != nil)
    {
      NSString *size = [imageDict objectForKey: @"size"];
      NSString *scale = [imageDict objectForKey: @"scale"];

      if ([size isEqualToString: @"32x32"] &&
          [scale isEqualToString: @"1x"])
        {
          filename = [imageDict objectForKey: @"filename"];
          break;
        }
    }

  // Copy icons to resource dir...
  NSString *productOutputDir = [NSString stringWithCString: getenv("PRODUCT_OUTPUT_DIR")];
  NSString *resourcesDir = [productOutputDir stringByAppendingPathComponent: @"Resources"];
  NSString *imagePath = [appIconDir stringByAppendingPathComponent: filename];
  NSString *destPath = [resourcesDir stringByAppendingPathComponent: filename];

  // NSLog(@"%@ -> %@", imagePath, resourcesDir);
  NSError *error = nil;

  [mgr copyItemAtPath: imagePath
               toPath: destPath
                error: &error];

  NSDebugLog(@"error = %@", error);

  return filename;
}

- (BOOL) processInfoPlistInput: (NSString *)inputFileName
                        output: (NSString *)outputFileName
{
  GSXCBuildContext *context = [GSXCBuildContext sharedBuildContext];
  NSString *settings = [context objectForKey: @"PRODUCT_SETTINGS_XML"];
  if(settings == nil)
    {
      NSMutableDictionary *dict = [NSMutableDictionary dictionary];
      NSString *inputFileString = [NSString stringWithContentsOfFile: inputFileName];
      NSString *outputFileString = nil;
      
      ASSIGNCOPY(outputFileString, inputFileString);
      
      // Get env vars...
      char **env = NULL;
      for (env = environ; *env != 0; env++)
        {
          char *thisEnv = *env;
          NSString *envStr = [NSString stringWithCString: thisEnv encoding: NSUTF8StringEncoding];
          NSArray *components = [envStr componentsSeparatedByString: @"="];
          [dict setObject: [components lastObject]
                   forKey: [components firstObject]];
        }
      
      // Replace all variables in the plist with the values...
      NSDebugLog(@"%@", dict);
      NSArray *keys = [dict allKeys];
      NSEnumerator *en = [keys objectEnumerator];
      NSString *k = nil;
      while ((k = [en nextObject]) != nil)
        {
          NSString *v = [dict objectForKey: k];
          outputFileString = [outputFileString stringByReplacingOccurrencesOfString: [NSString stringWithFormat: @"$(%@)",k]
                                                                         withString: v];
        }

      NSMutableDictionary *plistDict = [NSMutableDictionary dictionaryWithDictionary: [outputFileString propertyList]];
      NSString *filename = [self processAssets];
      if (filename != nil)
        {
          [plistDict setObject: filename forKey: @"NSIcon"];
        }
      
      [plistDict writeToFile: outputFileName
                  atomically: YES];
      
      NSDebugLog(@"%@", plistDict);
    }
  else
    {
      [settings writeToFile: outputFileName
                 atomically: YES
                   encoding: NSUTF8StringEncoding
                      error: NULL];      
    }
  return YES;
}

- (BOOL) build
{
  puts("=== Executing Resources Build Phase");
  NSFileManager *mgr = [NSFileManager defaultManager];
  NSString *productOutputDir = [NSString stringWithCString: getenv("PRODUCT_OUTPUT_DIR")];
  NSString *resourcesDir = [productOutputDir stringByAppendingPathComponent: @"Resources"];
  NSError *error = nil;
  NSString *productName = [self productName]; // @""; // [target productName];

  NSDebugLog(@"productName = %@", productName);
  
  // Pre create directory....
  [mgr createDirectoryAtPath:resourcesDir
       withIntermediateDirectories:YES
                  attributes:nil
                       error:&error];

  // Copy all resources...
  NSEnumerator *en = [files objectEnumerator];
  BOOL result = YES;
  id file = nil;
  while((file = [en nextObject]) != nil && result)
    {
      id fileRef = [file fileRef];

      if ([fileRef isKindOfClass: [PBXVariantGroup class]])
        {
          NSArray *children = [fileRef children];
          NSEnumerator *e = [children objectEnumerator];
          id child = nil;
          while ((child = [e nextObject]) != nil)
            {
              NSString *filePath = [child path];
	      NSDebugLog(@"FILEPATH = %@", filePath);
              NSString *resourceFilePath = [filePath stringByDeletingLastPathComponent];
              BOOL edited = NO;
              if ([mgr fileExistsAtPath: [child path]] == NO)
                {
                  edited = YES;
                  filePath = [productName stringByAppendingPathComponent: [child path]];
                }

              NSString *fileDir = [resourcesDir stringByAppendingPathComponent:
                                                  resourceFilePath];
              NSString *fileName = [filePath lastPathComponent];
              NSString *destPath = [resourcesDir stringByAppendingPathComponent: fileName];
              NSError *error = nil;
              BOOL copyResult = NO; 
              
              // If there is more than one path component...
              // then the intervening directories need to
              // be created.
              if([[filePath pathComponents] count] > 1)
                {
                  NSString *dirs = [filePath stringByDeletingLastPathComponent];
                  if (edited)
                    {
                      dirs = [dirs stringByReplacingOccurrencesOfString: productName withString: @""];
                    }
                  destPath = [resourcesDir stringByAppendingPathComponent: dirs];
                  destPath = [destPath stringByAppendingPathComponent: fileName];
                }
              
              NSDebugLog(@"\tCreate %@",fileDir);
              copyResult = [mgr createDirectoryAtPath: fileDir
                                withIntermediateDirectories: YES
					   attributes: nil
						error: &error];
              if (copyResult == NO)
                {
                  NSLog(@"\tFILE CREATION ERROR:  %@, %@", error, fileDir);
                }

              NSDebugLog(@"\t* Copy child %@  -> %@",filePath,destPath);
              puts([[NSString stringWithFormat: @"\t* Copy child resource %@ --> %@", filePath, destPath] cString]);
              copyResult = [mgr copyItemAtPath: filePath
                                        toPath: destPath
                                         error: &error];
              if (copyResult == NO)
                {
                  NSLog(@"\tERROR: %@, %@ -> %@", error, filePath, destPath);
                }
            }
          continue;
        }
      
      NSString *filePath = [file path];
      if ([mgr fileExistsAtPath: [file path]] == NO)
        {
          filePath = [productName stringByAppendingPathComponent: [file path]];
        }

      NSString *fileName = [filePath lastPathComponent];
      NSString *destPath = [resourcesDir stringByAppendingPathComponent: fileName];
      NSError *error = nil;
      BOOL copyResult = NO; 
      NSDebugLog(@"\tXXXX Copy %@ -> %@",filePath,destPath);
      puts([[NSString stringWithFormat: @"\t* Copy resource %@ --> %@",filePath,destPath] cString]);      
      copyResult = [mgr copyItemAtPath: filePath
                                toPath: destPath
                                 error: &error];
      if(!copyResult)
	{
	  NSDebugLog(@"\tCopy Error: %@ copying %@ -> %@",[error localizedDescription],
                     filePath, destPath);
	}
    }

  // Handle Info.plist....
  char *infoplist = getenv("INFOPLIST_FILE") == NULL ? "":getenv("INFOPLIST_FILE");
  NSString *inputPlist = [NSString stringWithCString:
                                     infoplist];
  if ([mgr fileExistsAtPath: inputPlist] == NO)
    {
      inputPlist = [inputPlist lastPathComponent];
    }
  
  NSString *outputPlist = [resourcesDir
                            stringByAppendingPathComponent: @"Info-gnustep.plist"];
  // NSLog(@"resourcesDir = %@ %s", resourcesDir, infoplist);
  [self processInfoPlistInput: inputPlist
                       output: outputPlist];

  // Move Base.lproj to English.lproj until Base.lproj is supported..
  NSString *baseLproj = [resourcesDir
                          stringByAppendingPathComponent: @"Base.lproj"];
  // NSString *engLproj =  [resourcesDir
  //                    stringByAppendingPathComponent: @"English.lproj"];
  // NSError *error = nil;
  //NSDirectoryEnumerator *den = [[NSDirectoryEnumerator alloc] initWithDirectoryPath: baseLproj
  //                                                        recurseIntoSubdirectories: NO
  //                                                                   followSymlinks: NO
  //                                                                     justContents: YES
  //for: self];

  NSArray *farr = [mgr directoryContentsAtPath: baseLproj];
  NSDebugLog(@"files in dir %@", farr);

  NSEnumerator *fen = [farr objectEnumerator];
  NSString *f = nil;

  while ((f = [fen nextObject]) != nil)
    {
      NSString *src = [baseLproj stringByAppendingPathComponent: f];
      NSString *dst = [resourcesDir stringByAppendingPathComponent: f];
      NSError *ferror = nil;      
      [mgr moveItemAtPath: src
                   toPath: dst
                    error: &ferror];
      if (ferror != nil)
        {
          NSLog(@"** Error while copying resource: %@", ferror);
        }
    }

  /*
  if (den !=nil)
    {
      NSString *fn = nil;
      
      while ((fn = [den nextObject]) != nil)
        {
          [mgr moveItemAtPath: baseLproj
                       toPath: engLproj
                        error: NULL];
          if (error != nil)
            {
              NSLog(@"** Error while copying resource: %@", error);
            }
        }
        } */
  
  
  puts("=== Resources Build Phase Completed");
  return result;
}

- (BOOL) generate
{
  GSXCBuildContext *context = [GSXCBuildContext sharedBuildContext];
  NSMutableArray *resources = [NSMutableArray arrayWithCapacity: [files count]];
  
  puts("=== Generating Resources Entries Build Phase");
  NSFileManager *mgr = [NSFileManager defaultManager];
  NSString *productName = [target productName];
  NSString *appName = [productName stringByDeletingPathExtension];
  
  // Copy all resources...
  NSEnumerator *en = [files objectEnumerator];
  BOOL result = YES;
  id file = nil;
  while((file = [en nextObject]) != nil && result)
    {
      id fileRef = [file fileRef];
      if ([fileRef isKindOfClass: [PBXVariantGroup class]])
        {
          NSArray *children = [fileRef children];
          NSEnumerator *e = [children objectEnumerator];
          id child = nil;
          while ((child = [e nextObject]) != nil)
            {
              NSString *filePath = [child path];
              BOOL edited = NO;
              if ([mgr fileExistsAtPath: [child path]] == NO)
                {
                  edited = YES;
                  filePath = [productName stringByAppendingPathComponent: [child path]];
                }

              puts([[NSString stringWithFormat: @"\tAdd child resource entry %@", filePath] cString]);
              [resources addObject: filePath];
            }
          continue;
        }
      
      NSString *filePath = [file path];
      if ([mgr fileExistsAtPath: [file path]] == NO)
        {
          filePath = [productName stringByAppendingPathComponent: [file path]];
        }

      puts([[NSString stringWithFormat: @"\tAdd resource entry %@",filePath] cString]);      

      [resources addObject: filePath];
    }

  // Handle Info.plist....
  char *infoplist = getenv("INFOPLIST_FILE") == NULL ? "":getenv("INFOPLIST_FILE");
  NSString *inputPlist = [NSString stringWithCString:
                                     infoplist];
  if ([mgr fileExistsAtPath: inputPlist] == NO)
    {
      inputPlist = [inputPlist lastPathComponent];
    }

  
  NSString *outputPlist = [NSString stringWithFormat: @"%@Info.plist",appName] ;
  [self processInfoPlistInput: inputPlist
                       output: outputPlist];

  // Move Base.lproj to English.lproj until Base.lproj is supported..
  // NSString *baseLproj =  @"Base.lproj/*";
  // NSString *engLproj =  @"English.lproj";
  // [resources addObject: engLproj];
  [resources addObject: outputPlist];
  
  [context setObject: resources forKey: @"RESOURCES"];
  puts("=== Resources Build Phase Completed");
  return result;
}

@end
